use std::process::Command;

use tauri::Manager;

use crate::app::APP;
use crate::errors::*;
use crate::state::AppState;


pub async fn launch(use_base: bool) -> Result<(), ExecError> {
    let state = APP.get().unwrap().state::<AppState>();
    // do all this in a block so we don't hold the lock any longer than necessary
    let mut cmd = {
        let config = state.config.read().await;
        let mut cmd = Command::new(&config.terminal.exec);
        cmd.args(&config.terminal.args);
        cmd
    };

    // similarly
    {
        let state = APP.get().unwrap().state::<AppState>();
        let app_session = state.session.read().await;
        let (base_creds, session_creds) = app_session.try_get()?;
        if use_base {
            cmd.env("AWS_ACCESS_KEY_ID", &base_creds.access_key_id);
            cmd.env("AWS_SECRET_ACCESS_KEY", &base_creds.secret_access_key);
        }
        else {
            cmd.env("AWS_ACCESS_KEY_ID", &session_creds.access_key_id);
            cmd.env("AWS_SECRET_ACCESS_KEY", &session_creds.secret_access_key);
            cmd.env("AWS_SESSION_TOKEN", &session_creds.token);
        }
    }

    match cmd.spawn() {
        Ok(_) => Ok(()),
        Err(e) if std::io::ErrorKind::NotFound == e.kind() => {
            Err(ExecError::NotFound(cmd.get_program().to_owned()))
        },
        Err(e) => Err(e.into()),
    }
}
